@@ -1151,7 +1151,7 @@ setMethod("c_hitas_cpp", signature=c("sdcProblem", "list"), definition=function(
 })
 
 setMethod("c_quick_suppression", signature=c("sdcProblem", "list"), definition=function(object, input) {
-  freq <- id <- sdcStatus <- NULL
+  freq <- id <- sdcStatus <- weights <- NULL
   verbose <- input$verbose
   pI <- g_problemInstance(object)
   indices <- g_partition(object)$indices
@@ -1167,8 +1167,8 @@ setMethod("c_quick_suppression", signature=c("sdcProblem", "list"), definition=f
   setnames(dat, vNames)
   dat[,id:=1:nrow(dat)]
   dat[,freq:=g_freq(pI)]
+  dat[,weights:=g_weight(pI)]
   dat[,sdcStatus:=g_sdcStatus(pI)]
-
   dimVars <- match(vNames, names(dat))
   nDims <- length(dimVars)
   freqInd <- match("freq", colnames(dat))
